from __future__ import print_function
import sys,struct,glob,array,numpy
import makeHistogram as mH

def timeTrigNet(inputParam, TimeList):
    """
    Time table make from TrigNET T0
    """

    if isinstance(inputParam,int):
	filename=mH.getFilePath(inputParam,'TrigNET')
    elif isinstance(inputParam,str) or isinstance(inputParam,unicode):
        filelist=glob.glob(inputParam)
        filename=filelist[0]

    if len(filename)>0:
	firstPID = -1 # local
	PID = 0 # local
	PIDb = -1 # local
	#nT0 = 0

	#--------------------------
	f = open(filename, "rb")
	while 1:
	    a=f.read(8)
	    if len(a)==0:
		break
	#--------------------------
            b=mH.decode_VNR_TrigNET(a)

            if b[1]==-999:
                continue

            if b[0]=='0x5b':

                PID = long(b[3])
                if firstPID == -1:
                    firstPID = PID
		    PIDb = PID - 1

	    else:
		if b[0]=='0x5c':
		    clk=str2clk(a[1:8])
		    T0=clk[0]+clk[1]+clk[2]
		    PIDb += 1
		    while ( PID > PIDb):
			T0b += 0.04
			TimeList.append(T0b)
			PIDb += 1
			#nT0 += 1
		    TimeList.append(T0)
		    #nT0 += 1
		    T0b = T0

	#--------------------------
	f.close()
	#--------------------------
    return TimeList


def c2l(c):
	return long(ord(c))

def str2clk(d):
	if 7==len(d):
	    sec=long((c2l(d[0])<<22)+(c2l(d[1])<<14)+(c2l(d[2])<<6)+(c2l(d[3])>>2))
	    subsec=float(((c2l(d[3])&3)<<13)+(c2l(d[4])<<5)+(c2l(d[5])>>3))/32768.
	    submsec=float(((c2l(d[5])&0x7)<<8)+c2l(d[6]))/40000000.
	    return sec,subsec,submsec

def getEventDataArrayTime(inputParam,detector="MWPC",pixels=(256,256),dio=-1, nT0start=0, nT0end=-1, initial_bin=10000.0, final_bin=38100.0):
    """
    Modified in 1703 ver. (s.kasai)
    used array.array to conserve memory
    """
    if nT0end == -1:
        nT0end = 2 ** 32  # nT0 end set to very large number

    print("dio in getEventDataArray", dio)

    dioSortingList=[]
    if dio != -1:
        dioSortingList = mH.processTrigNet(inputParam, dioSortingList, dio)

    T0Count = []
    T0Count = mH.createT0Count(dio, dioSortingList, T0Count)

    TimeList=[]
    TimeList = timeTrigNet(inputParam, TimeList)

    if isinstance(inputParam,int):
        filelist=mH.getFilePathNew(inputParam,detector)
    elif isinstance(inputParam,str) or isinstance(inputParam,unicode):
        filelist=glob.glob(inputParam)
    
    PX=pixels[0]
    PY=pixels[1]
    if dio == -1 or dio == 0:

        data=[[[array.array('I') for i in xrange(PX)] for j in xrange(PY)]]
    else:
        data=[[[array.array('I') for i in xrange(PX)] for j in xrange(PY)],
              [[array.array('I') for i in xrange(PX)] for j in xrange(PY)],
              [[array.array('I') for i in xrange(PX)] for j in xrange(PY)],
              [[array.array('I') for i in xrange(PX)] for j in xrange(PY)]]

    indexDIO=0
    t0indexorigin=-1
    iDIO = 0 # flag
    nDIO = 0 # count bet. each periods
    fDIO = 0 # flag for increasing an index
    nT0 = 0 # for dio=-1

    #nT0Range = 0L # for T0 range

    #func_name = "".join(['decode_',XXX,'_',detector])
    #func = globals()[func_name]

    #nT0_in_range = False # Flag if nT0 is in parameters set by option
    for filename in filelist:
        with open(filename, "rb") as f:
	    i1=0
	    i2=0

	    while 1:
		a=f.read(8)
		if len(a)==0:
		    break
		#b = func(a)
		b = mH.decode_VNR_MWPC(a)
		if b[1]==-999:
                    #AS Start 20160125 4 byte back for many bad data reduction.
                    f.seek(-4,1)
                    #AS End
                    continue

                if b[0]=='0xb7':
		    # check time  by s.k
		    #decode = struct.unpack('8B', a)
		    #t0count = (decode[7]<<23) | (decode[6]<<15) | (decode[5]<<7) | ((decode[4]&0xFE)>>1)
		    t0count = b[2]
		    if t0indexorigin < 0:
			t0indexorigin = t0count
		    neutime = TimeList[ t0count - t0indexorigin] - TimeList[0]
		    if ( neutime >= float( nT0end)):
			break
		    # nT0 range check (1610 AS)
		    # nT0Range+=1L
		    # if nT0Range > nT0end:
		    #    break
		    # elif nT0Range >= nT0start:
		    #    nT0_in_range = True
		    # nT0 range check end
		    if dio == -1:
			if neutime > float(nT0start):
			    nT0+=1
		    else:
			if len(dioSortingList)==0:
			    break

			if fDIO==1:
			    indexDIO+=1
			    nDIO = 0
			    fDIO = 0
			    if indexDIO >= len(dioSortingList):
				break

			PID = long(b[2])
                        #print '################### ', indexDIO, len(dioSortingList)
                        if dioSortingList[indexDIO][1]>PID:
                        #print 'before PID ', PID
                            iDIO = 0
                            nDIO = 0

                        elif dioSortingList[indexDIO][1]+dioSortingList[indexDIO][3]>=PID:
                            iDIO = 1
                            #print '*** ', PID, dioSortingList[indexDIO]

                        if dioSortingList[indexDIO][1]+dioSortingList[indexDIO][3]==PID:
                            fDIO = 1

		#elif b[0] == '0xb5' and nT0_in_range: # nT0 flag check added (1610 AS)
		elif b[0] == '0xb5':
                    if b[1]>=initial_bin and b[1]<=final_bin and b[2]<PX and b[3]<PY:
			t0time = TimeList[ t0count - t0indexorigin] - TimeList[0]
			neutime = t0time + float(b[1]) / 1000000.0
			neui = int( neutime)

			if ( neui >= nT0start):
			    if dio == -1:
				data[0][b[3]][b[2]].append(neui)
			    elif dio == 0:
				if iDIO == 1:
				    nDIO+=1
				    data[0][b[3]][b[2]].append(neui)
			    else:
				if iDIO == 1:
				    nDIO+=1
				    data[dioSortingList[indexDIO][0]][b[3]][b[2]].append(neui)


    if dio == -1:
	T0Count.append(nT0)

    datalist=[]

    if dio == -1 or dio == 0:
        dl = []
        dl=[[numpy.array(data[0][j][i]) for i in xrange(PX)] for j in xrange(PY)]
        datalist.append(dl)
    else:
        for k in [0,1,2,3]:
            dl = []
            dl=[[numpy.array(data[k][j][i]) for i in xrange(PX)] for j in xrange(PY)]
            datalist.append(dl)

    print(T0Count)
    return datalist, T0Count, neutime

################
def getEventDataArrayMulti(inputParam,detector="MWPC",pixels=(256,256),dio=-1, nT0start=0, nT0end=-1, nT0step=-1, initial_bin=10000.0, final_bin=38100.0):
    """
    Made in 1703 ver. (by s.kasai)
    used array.array to conserve memory
    """
    if nT0end == -1:
        nT0end = 2 ** 32  # nT0 end set to very large number

    print("dio in getEventDataArray", dio)

    dioSortingList=[]
    if dio != -1:
        dioSortingList = mH.processTrigNet(inputParam, dioSortingList, dio)

    TimeList=[]
    TimeList = timeTrigNet(inputParam, TimeList)

    if isinstance(inputParam,int):
        filelist=mH.getFilePathNew(inputParam,detector)
    elif isinstance(inputParam,str) or isinstance(inputParam,unicode):
        filelist=glob.glob(inputParam)
    
    PX=pixels[0]
    PY=pixels[1]
    if dio == -1 or dio == 0:

        data=[[[array.array('I') for i in xrange(PX)] for j in xrange(PY)]]
    else:
        data=[[[array.array('I') for i in xrange(PX)] for j in xrange(PY)],
              [[array.array('I') for i in xrange(PX)] for j in xrange(PY)],
              [[array.array('I') for i in xrange(PX)] for j in xrange(PY)],
              [[array.array('I') for i in xrange(PX)] for j in xrange(PY)]]

    indexDIO=0
    t0indexorigin=-1
    t0count = 0

    iDIO = 0 # flag
    # nDIO = 0 # count bet. each periods
    nextHisto = False # go next histogram
    fDIO = 0 # flag for increasing an index


    nT0 = 0 # for dio=-1
    nT0l = [0, 0, 0, 0] # for dio=0, or above
    T0Count = []
    datalist=[]

    #func_name = "".join(['decode_',XXX,'_',detector])
    #func = globals()[func_name]

    for filename in filelist:
        with open(filename, "rb") as f:

	    while 1:
		a=f.read(8)
		if len(a)==0:
		    break
		#b = func(a)
		b = mH.decode_VNR_MWPC(a)
		if b[1]==-999:
                    #AS Start 20160125 4 byte back for many bad data reduction.
                    f.seek(-4,1)
                    #AS End
                    continue

                if b[0]=='0xb7':
		    t0count = b[2]
		    if t0indexorigin < 0:
			t0indexorigin = t0count
		    neutime = TimeList[ t0count - t0indexorigin] - TimeList[0]
		    if ( neutime >= float( nT0end)):
			break
		    if ( nextHisto == True and nT0step > 0):
			nT0start += nT0step
			if dio == -1:
			    T0Count.append(nT0)
			    nT0 = 0
			else:
			    if dio == 0:
				T0Count.append( sum(nT0l))
			    else:
				T0Count.append( nT0l)
			    nT0l = [0,0,0,0]

			if dio == -1 or dio == 0:
			    dl = []
			    dl=[[numpy.array(data[0][j][i]) for i in xrange(PX)] for j in xrange(PY)]
			    datalist.append(dl)
			    data=[[[array.array('I') for i in xrange(PX)] for j in xrange(PY)]]
			else:
			    for k in [0,1,2,3]:
				dl = []
				dl=[[numpy.array(data[k][j][i]) for i in xrange(PX)] for j in xrange(PY)]
				datalist.append([dl])
			    data=[[[array.array('I') for i in xrange(PX)] for j in xrange(PY)],
				  [[array.array('I') for i in xrange(PX)] for j in xrange(PY)],
				  [[array.array('I') for i in xrange(PX)] for j in xrange(PY)],
				  [[array.array('I') for i in xrange(PX)] for j in xrange(PY)]]
			nextHisto = False

		    if dio == -1:
			if neutime > float(nT0start):
			    nT0+=1
		    else:
			if len(dioSortingList)==0:
			    break

			if fDIO==1:
			    indexDIO+=1
			    fDIO = 0
			    if indexDIO >= len(dioSortingList):
				break

			PID = long(b[2])
			if dioSortingList[indexDIO][1]>PID:
			    iDIO = 0

			elif dioSortingList[indexDIO][1]+dioSortingList[indexDIO][3]>=PID:
			    iDIO = 1
			    T0Count[dioSortingList[indexDIO][0]] += 1

			if dioSortingList[indexDIO][1]+dioSortingList[indexDIO][3]==PID:
			    fDIO = 1

		elif b[0] == '0xb5' and t0indexorigin >= 0:
		    t0time = TimeList[ t0count - t0indexorigin] - TimeList[0]
		    if ( nT0step > 0 and (t0time + 0.04) >= float( nT0start+nT0step)):
			nextHisto = True
		    if b[1]>=initial_bin and b[1]<=final_bin and b[2]<PX and b[3]<PY:
			neutime = t0time + float(b[1]) / 1000000.0

			if ( neutime >= float(nT0start)):
			    if dio == -1:
				data[0][b[3]][b[2]].append(b[1])
			    elif dio == 0:
				if iDIO == 1:
				    #nDIO+=1
				    data[0][b[3]][b[2]].append(b[1])
			    else:
				if iDIO == 1:
				    #nDIO+=1
				    data[dioSortingList[indexDIO][0]][b[3]][b[2]].append(b[1])

    if dio == -1:
	if ( nT0 > 0):
	    T0Count.append(nT0)
	    dl = []
	    dl=[[numpy.array(data[0][j][i]) for i in xrange(PX)] for j in xrange(PY)]
	    datalist.append(dl)
    else:
	if ( sum(nT0l) > 0):
	    if ( dio == 0):
		T0Count.append( sum(nT0l))
		dl = []
		dl=[[numpy.array(data[0][j][i]) for i in xrange(PX)] for j in xrange(PY)]
		datalist.append(dl)
	    else:
		T0Count.append( nT0l)
		for k in [0,1,2,3]:
		    dl = []
		    dl=[[numpy.array(data[k][j][i]) for i in xrange(PX)] for j in xrange(PY)]
		    datalist.append([dl])

    print(T0Count)
    return datalist, T0Count



import Manyo.UtsusemiEventConv2Q as mu

import Manyo
import os
import sys

#wfile = "xml/wfile.xml"
wfile = "xml/wfile_all.xml"
dfile = "xml/dfile.xml"
xfile = "XtalParam6.xml"
daqId = 0
modNo = 0
runNo = 373
withWeight=True
rotInfo="Y:15"
isReadEventData = True
if len(sys.argv)==2:
    flag = sys.argv[1]
    flag = flag.upper()
    if flag=="TRUE":
        isReadEventData = True
    elif flag=="FALSE":
        isReadEventData = False

if isReadEventData:
    if withWeight:
        os.system("rm -rf data/SIK000373_*.evt")
    else:
        os.system("rm -rf data2/SIK000373_*.evt")

tt=mu.UtsusemiEventDataConverterNeunet2QEvent()
tt.SetEi( 45.56, -4.0, 40.0, 18030.0 )
if not tt.SetTextMaskInfo("mask.txt"):
    raise UserWarning,"--- Failed SetTextMaskInfo"
tt.SetWithWeightOutput( withWeight )
tt.SetXtalParamFile( xfile, rotInfo )
if withWeight:
    tt.SetOutputPath( "/home/yinamura/dev/works/180319NewEvt/data" )
else:
    tt.SetOutputPath( "/home/yinamura/dev/works/180319NewEvt/data2" )
tt.LoadParamFiles( str(wfile), str(dfile), "" )
tt.SetDataPath( "/data","", runNo )
#tt.DumpEventToText( "data/SIK000002_00_000.evt", "data/SIK000002_00_000.txt" )
flist = Manyo.MakeStringVector()
for daqId in [0,1]:
    for modNo in range(12):
        if isReadEventData:
            tt.ReadEventData( daqId, modNo )
        flist.append( "data/SIK%06d_%02d_%03d.evt"%(runNo, daqId, modNo) )
        print("---- convert SIK%06d_%02d_%03d.evt"%(runNo, daqId, modNo))
eca=tt.Slice2d( flist, xfile )

import vis.M2Plot as mp
p=mp.M2Plot(eca)



import os
import Manyo.Utsusemi as mu

class SetESiniFile:
    def __init__(self,ESinifile):
        if os.path.exists(ESinifile):
            self.ESinifile = ESinifile
        else:
            self.ESinifile = os.path.join(mu.UtsusemiEnvGetInstDir(),"ana","ElasticScanGUI",ESinifile)
        self.readESinifiles()
        self.commentheader = "ESGUIBase -->"
    def readESinifiles(self):
        f = open(self.ESinifile)
        _lines = f.readlines()
        f.close()
        
        self.ESini = {}
        for line in _lines:
            tt = line.split()
            if tt[0][0] != "#":
                self.ESini.update({tt[0]:tt[1]})
    
    def setparams(self,paramname,paramvalue):
        print(paramname, paramvalue)
        if paramname in self.ESini:
            self.ESini[paramname] = paramvalue
            print ("SetESiniFile -->", paramname, "is set as ", self.ESini[paramname],".")
        else:
            print ("SetESiniFile -->", paramname, "is not found in ", self.ESinifile,".")
            self.ESini.update({paramname: paramvalue})
            print ("SetESiniFIle -->", paramname, "is added as ", self.ESini[paramname], ".")


    def writeESinifiles(self):
        _savestring = "#generated by SetESiniFile\n"
        for k, v in self.ESini.items():
            _savestring += "%s  %s\n" %(k, v)
        
        _filename = "ESGUIParams.ini"
        f = open(_filename,"w")
        f.write(_savestring)
        f.close
        
        cwdpath = os.getcwd()+"/"+_filename
        #inipath = "/home/dnaadmin/ana/DNA/setup/"+_filename
        inipath = os.path.join(mu.UtsusemiEnvGetUserDir(), "ana", "DNA", "setup", _filename)
        if os.path.exists(inipath):
            os.remove(inipath)
            print (self.commentheader, "inifile:", cwdpath, " is removed")
        os.symlink(cwdpath,inipath)
        print (self.commentheader, "inifile:", cwdpath, " is updated")
        



#!/usr/bin/python
# -*- coding: utf-8 -*-
"""
MPlotJupyter.py
uGao.MPlot I/F on Jupyter and I/F of the dark mode
"""
from __future__ import print_function
import os
import Manyo
import six

from uGao.uGaoUtil import IFEvtProp, ConvertEC, TraceAtt
from uGao.FastPlotQt import Chart, PlotData
import matplotlib
ISJUPYTER = False
if matplotlib.get_backend() == "inline":
    ISJUPYTER = True
    import matplotlib.pyplot as plt
else:
    from matplotlib.figure import Figure
    from matplotlib.backends.backend_agg import FigureCanvasAgg as FigCanvas

"""
CUI to control FastPlot on Jupyter or with non-X-server environment

Sample code:
    import Manyo
    import uGao.MPlotJupyter as FP

    ec = Manyo.ElementContainer()
    ec.Add("X", [1.0, 2.0, 3.0, 4.0, 5.0])
    ec.Add("Y", [1.0, 2.0, 0.1, 1.0])
    ec.Add("E", [1.0, 1.0, 1.0, 1.0])
    ec.SetKeys("X", "Y", "E")

    ec2 = ec.Mul(2.0)

    p = FP.MPlotJupyter()
    p.AddData([ec, ec2])

    p.SetYScale(False, 0.01, 10)
    p.SetLog(True)
    p.SetXScale(False, -1, 10)
    #p.RemoveData(1)
    p.SetPlotParam(0, ls="-", lc="k", lw=9)
    p.SetMainTitle("Run No: 5515")
    p.SetSubTitle("This is test!")
    p.SetXLabel("Index X")
    p.SetYLabel("Intensity")

    # Save Plot figure as a file
    p.Save("./test.png")
    # Plot on Jupyter
    p.Show()
"""

#######################################
#  MPlotJupyter
#######################################
class MPlotJupyter(object):
    def __init__(self, data=None, order=1, fTitle=None, sizeX=8.0, sizeY=5.0, dpi=90):
        """
        """
        self.order = order
        self.dpi = dpi
        # データオブジェクトを設定
        self.dataobj = PlotData(self, None, order)
        # キャンバスを作成
        if ISJUPYTER:
            print("is Jupyter")
            self.plt = plt
            self.fig = plt.figure()
            self.canvas = self.fig.canvas
        else: # normal inDark mode
            print("is Dark mode ")
            self.fig = Figure((sizeX, sizeY), dpi)
            self.canvas = FigCanvas(self.fig)

        Chart(self, self.dataobj)

        # プロッタが公開しているインターフェイスクラスのインスタンスを取得
        self.ifi = IFEvtProp(order)

        if data is not None:
            self.AddData(data)

    ###################################################
    def Show(self):
        """
        """
        if ISJUPYTER:
            self.plt.show()
        else:
            pass
    ###################################################
    def AddData(self, data):
        """
        データ追加イベント発行
        @param  data (ElementContainer or list of ElementContainer)
        @retval None
        """
        ce = ConvertEC()
        lstDat = []
        # リストか
        if isinstance(data, list):
            # リスト中のエレメントコンテナ毎に変換

            for ec in data:
                ret = ce.ConvertEC(ec)
                if ret is None:
                    print("\nArgument is not acceptable.")
                    return
                # Manyo データからPython データタイプへ変換
                lstDat.append(ret)
        # 単一データ
        else:
            # Manyo データからPython データタイプへ変換
            ret = ce.ConvertEC(data)
            if ret is None:
                print("\nArgument is not acceptable.")
                return
            lstDat.append(ret)

        self.ifi.NotifyEvent(self, 'add', lstDat)

    ###################################################
    def Remove(self, fignum):
        """
        データ削除イベント発行
        @param  fignum (int) index of figure
        @retval None
        """
        # データ数取得
        dno = self.ifi.GetProperty("datanum")
        # 指定されたデータ数は、実際のデータ数以内か
        if dno >= int(fignum):
            # データ削除イベント発行
            self.ifi.NotifyEvent(self, "remove", int(fignum))

    ###################################################
    def ChangeDataOrder(self, swap1, swap2):
        """
        データ順番変更イベント発行
        @param  swap1 (int) figure index to swap
        @param  swap2 (int) figure index to swap
        @retval None
        """
        # データ数取得
        dno = self.ifi.GetProperty("datanum")
        # 指定されたデータ数は、実際のデータ数以内か
        if dno >= int(swap1) and dno >= int(swap2):
            # データ順番変更イベント発行
            self.ifi.NotifyEvent(self, "change", (int(swap1), int(swap2)))
        else:
            # 否定応答を返信
            return "NACK\n"

    ###################################################
    def ChangeLogMode(self, axis, flag):
        """
        ログモード変更
        @param axis (str) "X" or "Y" for applying log scale
        @retval None
        """
        if axis.upper() == "X":
            self.ifi.NotifyEvent(self, "logx", flag)
        elif axis.upper() == "Y":
            self.ifi.NotifyEvent(self, "log", flag)

    #########################################
    def SetLog(self, flag=None):
        """
        Y-axisログモード設定
        @param  flag (bool) Log scale for Y axis
        @retval None
        """
        if isinstance(flag, bool):
            self.ChangeLogMode("Y", flag)
    #########################################
    def SetLogX(self, flag=None):
        """
        X-axisログモード設定
        @param  flag (bool) Log scale for X axis
        @retval None
        """
        if isinstance(flag, bool):
            self.ChangeLogMode("X", flag)
    #########################################
    def SetLogY(self, flag=None):
        """
        Y-axisログモード設定
        @param  flag (bool) Log scale for Y axis
        @retval None
        """
        if isinstance(flag, bool):
            self.ChangeLogMode("Y", flag)

    ###################################################
    def SetXScale(self, isAuto, min_X, max_X):
        """
        Set scale for X-axis
        @param isAuto (bool) True means Auto scale for X axis
        @param min_X (float) minimum value of X scale when isAuto is False
        @param max_X (float) maximum value of X scale when isAuto is False
        @retval None
        """

        if isAuto:
            # スケール変更(自動)イベント発行
            self.ifi.NotifyEvent(self, "xscale", (True,))
        else:
            # マニュアルスケール範囲、X軸範囲または Y軸範囲
            self.ifi.NotifyEvent(
                self, "xscale", (False, float(min_X), float(max_X)))
    ###################################################
    def SetYScale(self, isAuto, min_Y, max_Y):
        """
        Set scale for Y-axis
        @param isAuto (bool) True means Auto scale for Y axis
        @param min_Y (float) minimum value of Y scale when isAuto is False
        @param max_Y (float) maximum value of Y scale when isAuto is False
        @retval None
        """
        if isAuto:
            # スケール変更(自動)イベント発行
            self.ifi.NotifyEvent(self, "yscale", (True,))
        else:
            # マニュアルスケール範囲、X軸範囲または Y軸範囲
            self.ifi.NotifyEvent(
                self, "yscale", (False, float(min_Y), float(max_Y)))

    #########################################
    def SetOffset(self, xoffset=None, yoffset=None):
        """
        オーバレイ時のオフセット指定
        @param  xoffset (int)  X軸オフセット
        @param  yoffset (int)  Y軸オフセット
        @retval 無し
        """
        # オフセットを入力していない場合
        if xoffset is None or yoffset is None:
            self._EmptyName('XOffset and YOffset')
        # 整数変換できるか
        else:
            try:
                xo = int(xoffset)
                yo = int(yoffset)
            except:
                print("\nError!!  Arguments must be integer.")

            # Xオフセット範囲チェック
            if xo < 0 or xo > 200:
                print("\nError!!  Offset must be between 0 to 200.")
                return
            # Yオフセット範囲チェック
            if yo < 0 or yo > 200:
                print("\nError!!  Arguments must be between 0 to 200.")
                return
            # コマンド作成
            self.ifi.NotifyEvent(
                self, "offset", (xoffset, yoffset))

    #########################################
    def SetPlotParam(self, dnum=None, **kargs):
        """
        プロットパラメータ指定
        @param  dnum     データNo  または "all"
        @param  **kargs   キーワード引数
                ls        線種　['-', ':','--', '-.','']
                hs        ヒストグラムスタイル　True or False
                lw        線幅　1〜10
                lc        線色　['*', 'b', 'g', 'r', 'c', 'm', 'y', 'k']
                mk        マーカーの種類  ['', '.','o','s','v','d','+','x']
                ms        マーカーのサイズ　1〜10
                mc        マーカーの色　['*', 'b', 'g', 'r', 'c', 'm', 'y', 'k']
                eb        エラーバー指定    True or False
                es        エラーバーのキャップサイズ　1〜10
                sc        エラーバーの色　['*', 'b', 'g', 'r', 'c', 'm', 'y', 'k']
        @retval 無し
        """
        # データ番号を入力しなかった場合
        if dnum is None:
            self._EmptyName('Data Number')
        else:
            # 整数変換できるか
            try:
                dno = int(dnum)
            except:
                print('\nError data number!! The argument 1 must be integer.')
                return
            # キーワード引き数の数を確認
            if len(kargs) == 0:
                print("\nError!!  It needs at least one keyword.")
                print(
                    'Available keywords are "ls", hs", "lw", "lc", "mk", "ms", "mc", "eb", "es" and "ec".')
                return
            print(kargs)
            # キーワードを取得
            for key in kargs:
                # 線種の指定か
                if key == "ls":
                    if not self._CheckValue(TraceAtt.linestyle, key, kargs[key]):
                        return
                # ヒストグラムスタイル指定か
                elif key == "hs":
                    if type(kargs[key]) != bool:
                        print('\nError!! "{}" requires True/False'.format(key))
                        return
                # 線幅の指定か
                elif key == "lw":
                    if not self._CheckNum(key, kargs[key]):
                        return
                # 線色の指定か
                elif key == "lc":
                    if not self._CheckValue(TraceAtt.color, key, kargs[key]):
                        return
                # マーカの指定か
                elif key == "mk":
                    if not self._CheckValue(TraceAtt.marker, key, kargs[key]):
                        return
                # マーカーサイズの指定か
                elif key == "ms":
                    if not self._CheckNum(key, kargs[key]):
                        return
                # マーカー色の指定か
                elif key == "mc":
                    if not self._CheckValue(TraceAtt.color, key, kargs[key]):
                        return
                # エラーバー指定か
                elif key == "eb":
                    if type(kargs[key]) != bool:
                        print('\nError!! "{}" requires True/False'.format(key))
                        return
                # キャップサイズの指定か
                elif key == "es":
                    if not self._CheckNum(key, kargs[key]):
                        return
                # エラーバー色の指定か
                elif key == "ec":
                    if not self._CheckValue(TraceAtt.color, key, kargs[key]):
                        return
                # キーワードエラー
                else:
                    print("\nError Keyword !!")
                    print(
                        'Available keywords are "ls", "lw", "lc", "mk", "ms", "mc", "eb", "es" and "ec".')
                    return

            self.ifi.NotifyEvent(self, "plotparam", (dno, kargs))
    #########################################
    def _CheckValue(self, lst, key, value):
        """
        指定されたキーの値をチェック
        @param  lst 　有効な値のリスト
        @param  key   キー文字列
        @param  value 値の文字列
        @retval True/False
        """
        # 指定された値はリスト中に存在するか
        if value in lst:
            # リスト中の値の順番を探す
            i = 0
            for item in lst:
                # リスト中にあるか
                if item == value:
                    return True
                i += 1

        # 不正な値だったなら
        else:
            strMsg = "\nError value : Avairable for {} are ".format(key)
            # 指定可能な値を列挙
            # 最後の2個の値を除き、カンマ付きで列挙
            for i in range(len(lst) - 2):
                item = '"%s", ' % lst[i]
                strMsg = strMsg + item
            # 最後の2個を連結
            i += 1
            j = i + 1
            item = '"%s" and "%s"' % (lst[i], lst[j])
            strMsg = strMsg + item
            print(strMsg)
            return False
    #########################################
    def _CheckNum(self, key, value):
        """
        指定されたキーの値をチェック
        @param  key   キー文字列
        @param  value 値
        @retval True/False
        """
        if type(value) is not int:
            print('\nError!!  Value for "%s" must be integer.' % key)
            return False
        if value < 1 or value > 10:
            print('\nError!! Value for "%s" must be between 1 to 10.' % key)
            return False

        return True
    #########################################
    def _EmptyName(self, name):
        """
        引数を入力しなかった場合入力を促すメッセージ表示
        @param name 入力項目
        @retval 無し
        """
        print("\nMissing argument error!! Input %s." % name)

    #########################################
    def Save(self, fname=None):
        """
        画像ファイルの保存
        @param  ファイル名
        @retval 無し
        """
        # 引数無し時のプロテクト
        if fname is None:
            self._EmptyName('filename')
            return
        try:
            six.u(fname)
        except:
            # 全角文字が含まれている場合エラーとする
            self._AsciiName('Filename')
            return
        work = fname.split(os.sep)
        print(work)
        # ファイル名取り出し
        filename = os.path.basename(fname)
        path, ext = os.path.splitext(filename)
        # 拡張子を除くファイル名の長さは1以上か
        if len(path) > 1:
            # サポートしている拡張子か
            if ext == ".png" or ext == ".ps" or ext == ".eps":
                self.canvas.print_figure(fname)
            else:
                print("\nError!!  Extention must be png, ps, or eps.")
        else:
            print("\nError!! Invalid file name.")

    #########################################
    def LoadFromText(self, filepath):
        """
        Load data from text file saved as MPlot
        @param filepath (str) path to file to load
        @retval None
        """
        if os.path.exists(filepath):
            self.ifi.NotifyEvent(self, "loaddatafromtext", filepath)
        else:
            raise UserWarning(
                "MPlot::LoadFromText >>> Not found such file={}".format(filepath))
    #########################################
    def SaveAsText(self, filepath, isHist=False, useMask=False, maskVal=None):
        """
        Save data as text file
        @param filepath (str) path to file
        @param isHist (bool) save text as histogram
        @param useMask (bool) Use mask value user defines
        @param maskVal (float) mask value user defines
        @retval None
        """
        p_isHist = "FALSE"
        p_useMask = "FALSE"
        p_maskVal = "NONE"
        if useMask:
            p_useMask = "TRUE"
            if maskVal is not None:
                p_maskVal = "%g".format(float(maskVal))

        self.ifi.NotifyEvent(self, "savedataastext", (filepath, p_isHist, p_useMask, p_maskVal))

    #########################################
    def SetXLabel(self, xlabel=None):
        """
        X軸のラベル設定
        @param  xlabel (string) word for X axis label
        @retval None
        """
        if xlabel is None or xlabel == "":
            self._EmptyName('xlabel')
        else:
            try:
                # 半角英数字を入力したか
                six.u(xlabel)
            except:
                self._AsciiName('xlabel')
            else:
                # コマンド作成
                self.ifi.NotifyEvent(self, "scalelabel", (xlabel, "*"))
    #########################################
    def SetYLabel(self, ylabel=None):
        """
        X軸のラベル設定
        @param  ylabel (string) word for Y axis label
        @retval None
        """
        if ylabel is None or ylabel == "":
            self._EmptyName('ylabel')
        else:
            try:
                # 半角英数字を入力したか
                six.u(ylabel)
            except:
                self._AsciiName('ylabel')
            else:
                # コマンド作成
                self.ifi.NotifyEvent(self, "scalelabel", ("*", ylabel))

    #########################################
    def SetMainTitle(self, main=None):
        """
        メインタイトル設定
        @param  main (str) Main title word
        @retval None
        """
        if main is None or main == "":
            self._EmptyName('MainTitle')
        else:
            try:
                # 半角英数字を入力したか
                six.u(main)
            except:
                self._AsciiName('MainTitle')

            else:
                # コマンド作成
                self.ifi.NotifyEvent(self, "title", (main, "*"))
    #########################################
    def SetSubTitle(self, sub=None):
        """
        サブタイトル設定
        @param  sub (str) Sub title word
        @retval None
        """
        if sub is None or sub == "":
            self._EmptyName('SubTitle')
        else:
            try:
                # 半角英数字を入力したか
                six.u(sub)
            except:
                self._AsciiName('SubTitle')
            else:
                # コマンド作成
                self.ifi.NotifyEvent(self, "title", ("*", sub))

    #########################################
    def _AsciiName(self, name):
        """
        Ascii文字入力を促すメッセージ表示
        @param name (string) The name of key
        @retval None
        """
        print("\nCode error!! %s must be ASCII string." % name)



#!/usr/bin/python

import time
import optparse

import Manyo
#import MultiDataLevmar as Adv
import Adv

import OptionSelector
import PlotNorm
import PlotFittingResult

###############################################################################

############            command line option            ############
parser=optparse.OptionParser()
parser.add_option("-i", "--input", action="store", type="string", dest="inputFile", help="input file name", metavar="FILE")
options, args = parser.parse_args()
if not options.inputFile:
    parser.error("no input file specified.")

############            read source file            ############
io=Manyo.NeXusFileIO()
src = io.ReadElementContainerArray(options.inputFile)
print src.PutSize(), "element container(s) in the read array."

src.PutHeaderPointer().Dump()
for i in range(src.PutSize()):
    #src.PutPointer(i).PutHeader().dump()
    src.PutPointer(i).dump()

domainList=[]
for i in range(src.PutSize()):
    ec=src.Put(i)
    domain=Adv.Domain(ec, ec.Put(ec.PutXKey()).front(), ec.Put(ec.PutXKey()).back())
    print "i=", i, " domain: (", domain.getLowerBound(), ", ", domain.getUpperBound(), ")"
    domainList.append(domain)

method=Adv.MultiDataLevmar()
paramSet=method.setDefaultParam(src)
paramSet.dump()

paramSet.replace(Adv.MultiDataLevmar.CONSTRAIN,          Adv.MultiDataLevmar.NO_CONSTRAIN)
paramSet.replace(Adv.MultiDataLevmar.USE_NUMERICAL_DIFF, Adv.MultiDataLevmar.DEFAULT_USE_NUMERICAL_DIFF)
paramSet.replace(Adv.MultiDataLevmar.DIFF_METHOD,        Adv.MultiDataLevmar.DEFAULT_DIFF_METHOD)
paramSet.replace(Adv.MultiDataLevmar.USE_DATA_WEIGHTS,   Adv.MultiDataLevmar.DEFAULT_USE_DATA_WEIGHTS)
paramSet.replace(Adv.MultiDataLevmar.MAX_ITERATIONS,     Adv.MultiDataLevmar.DEFAULT_MAX_ITERATIONS)
paramSet.replace(Adv.MultiDataLevmar.OUTPUT_INTERVAL,    Adv.MultiDataLevmar.DEFAULT_OUTPUT_INTERVAL)

functions=[]
for i in range(src.PutSize()):
    #parser=Adv.FuncParser("g g g")
    #fl=parser.parse()
    functions.append("g g g")

paramSet.add(Adv.MultiDataLevmar.FUNCTIONS, functions);

fittingParams=Manyo.VrctorTool.MakeVectorDouble();
fittingParams.push_back( 60000.0)
fittingParams.push_back(     3.5)
fittingParams.push_back(     1.5)
fittingParams.push_back( 60000.0)
fittingParams.push_back(     6.0)
fittingParams.push_back(     3.0)
fittingParams.push_back( 50000.0)
fittingParams.push_back(    12.0)
fittingParams.push_back(     3.5)
fittingParams.push_back(     1.0)
paramSet.add(Adv.MultiDataLevmar.PARAMETERS_VALUES, fittingParams);

paramSet.dump()

print "check parameters: ", method.checkParam(src, domainList, paramSet)

method.toInnerForm(src. domainList, paramSet)
method.fit()

while method.ifFitting():
    sleep(1)

method.eval();
result=Manyo.ElementContainerArray()
method.toElementContainer(src, result)
result.PutHeader.Dump()
print result.PutSize(), "element container(s) in the result array."
for i in range(result.PutSize()):
    result.Put(i).PutHeader().Dump()

fittedParamSet = method.getFittedParam()
fittedParamSet.dump()

outputIO = Manyo.NeXusFilIO()
outputIO.Write(result, "fittedArray.data", "tanimori")



#!/usr/bin/python3
# -*- coding: utf-8 -*-

import utsusemi.SAS.ana.Reduction.QSpaceBaseCommands as QBC

# special reserved word for commands history
HISTORY = None

# reserved words for return value in this.
DAT = None
DATList = None
ecs = None
EC = None
ret = None
PyObj = None
Iq = None
Tr = None

########################################
def ConvertQSpace( dat=DAT, InVec="0.0 0.0 1.0" ):
    """
    Convert Data to Q space EventData to produce histogram
    ConvertQSpace(dat=DAT, InVec="0.0 0.0 1.0")

    @param  dat    (ElementContainerMatrix)
    @param  InVec  (string) vector of the incident beam
    @retval None
    """
    QBC.ConvertQSpace( dat, InVec )

########################################
def ConvertQSpaceRPMTCenterShift(dat=DAT, DET=151, PIX=151):
    """
    Convert Data to Qspace on RPMT using shifted center position.
    The center position is at center point of the pixel given by DET and PIX.
    If DET or PIX is minus, for example -1, then this function calculates with the original position.

    ConvertQSpaceRPMTCenterShift(dat, DET, PIX)

    @param dat (ElementContainerMatrix) Target Data
    @param DET (int) Detect ID, or minus value (-1) to use original position
    @param PIX (int) Pixel No, or minus value (-1) to use original position
    @retval None
    """
    QBC.ConvertQSpaceRPMTCenterShift(dat, DET, PIX)

########################################
def ConvertQSpaceWithGravity( dat=DAT, params="default" ):
    """
    Convert Data to Q space EventData to produce histogram with gravity effect
    ConvertQSpaceWithGravity(dat=DAT, params="0.0")
    py = a+b*(lambda)^2

    params = "default" <- default : Using DetectorInfo/sas/rpmt parameters
    params = "150.093 -0.14218 136.9" <- a, b, px
    params = "0.1562 1.0 208.5 188.35" <- b, lambda, px, py@lambda

    @param  dat    (ElementContainerMatrix)
    @param  params (string) "default" or "<a> <b> <px>" or "<b> <lambda> <px> <py>"
    @retval None
    """
    QBC.ConvertQSpaceWithGravity( dat, params )

########################################
def QzProjection( dat=DAT, QxRange="-1.0 1.0 0.005", QyRange="-1.0 1.0 0.005", QzRange="-0.5 0.0", LambdaRange="0.7 7.6", Bank="All" ):
    """
    Makes Qx-Qy map data integrated in a Qz range.
    QzProjection( dat=DAT, QxRange="-1.0 1.0 0.005", QyRange="-1.0 1.0 0.005", QzRange="-0.5 0.0" )

    @param  dat   (ElementContainerMatrix) Data after ConvertToQSpace
    @param  QxRange   (string)  "Qx_min Qx_max Qx_bin"
    @param  QyRange   (string)  "Qy_min Qy_max Qy_bin"
    @param  QzRange   (string)  "Qz_max Qz_bin"
    @param LambdaRange(string)  "Lambda_min, Lambda_max"
    @param  Bank      (string)  "All" or "<BankName> <BankName> ..."
    @retval ECA   (ElementContainerArray)
    """
    return QBC.QzProjection( dat, QxRange, QyRange, QzRange, LambdaRange, Bank )

########################################
def SectorAverageByAngles( dat=DAT, Banks="SM16-SM14-SD-SU", Qrange="0.01 2.0 0.001 dQ/Q", LamRange="0.7 0.8", AzimAng="0.0 0.0", PolAng="0.0 0.0", QLamFile="None/QLambdaRange" ):
    """
    Convert Data to Q space EventData to produce histogram
    SectorAverageByAngles( dat=DAT, Banks="SM16-SM14-SD-SU", Qrange="0.01 2.0 0.001 dq/Q", LamRange="0.7 0.8", AzimAng="0.0 0.0", PolAng="0.0 0.0" )

    @param  dat  (ElementContainerMatrix)
    @param  Banks    (string)  "SM16-SD-SU" means these banks are unified in one Iq, "SM SD SU" means separated Iq
    @param  Qrange   (string)  Q range and type of Qbin "Qmin Qmax deltaQ type" type must be dQ/Q or dQ
    @param  LamRange (string)  Lambda range "Lambda_min Lambda_max"
    @param  AzimAng  (string)  "-20.0 20 170 -170"
    @param  PolAng   (string)  "0 90 0 40"
    @param  QLamFile (string)  filename of Q-Lambda range info. If None, no use. (read QLambdaRange_SM.txt for SM bank)
    @retval Iq       (list of ElementContainer)
    """
    return QBC.SectorAverageByAngles( dat, Banks, Qrange, LamRange, AzimAng, PolAng, QLamFile )

########################################
def MakeRegionOfSectAve( dat=DAT, Bank="SM", Qrange="0.01 2.0 0.001 dQ/Q", LamRange="0.7 7.6 0.1", AzimAng="0.0 0.0", PolAng="0.0 0.0", ErrTh="-1.0 e", OutFile="QLambdaRange", TwoThetaRange="0.0-180.0" ):
    """
    Makes Lamda slice SectorAverage
    MakeRegionOfSectAve( dat=DAT, Bank="SM", Qrange="0.01 2.0 0.001 dQ/Q", LamRange="0.7 7.6 0.1", AzimAng="0.0 0.0", PolAng="0.0 0.0", ErrTh=1.0, OutFile="QLambdaRange" )

    @param  dat  (ElementContainerMatrix)
    @param  Bank     (string)  Target bank. Only one bank can be accepted.
    @param  Qrange   (string)  Q range and type of Qbin "Qmin Qmax deltaQ type" type must be dQ/Q or dQ
    @param  LamRange (string)  Lambda range "Lambda_min Lambda_max"
    @param  AzimAng  (string)  "-20.0 20 170 -170"
    @param  PolAng   (string)  "0 90 0 40"
    @param  ErrTh    (string)   "sh type" sh : threshold of error bar to decide useless region (ErrTh<=0.0 means no decision), type="e" or "ie"
    @param  OutFile  (string)  output file name for Q-Lambda range for target bunk (filename = QLambdaRange_SM.xml)
    @param  TwoThetaRange (string)
    @retval IQLam    (ElementContainerArray)
    """
    return QBC.MakeRegionOfSectAve( dat, Bank, Qrange, LamRange, AzimAng, PolAng, ErrTh, OutFile, TwoThetaRange )

########################################
def GetAzimuthProfile(dat=DAT, Qcenter=0.036, dQ=0.002, azim_bin=1.0, azim_range="0.0 360.0", lambda_range="0 0"):
    """
    Calculates Azimuth Profile from DataReduction

    @param dat          (ElementContainerMatrix)
    @param Qcenter      (float)
    @param dQ           (float)
    @param azim_bin     (float)
    @param azim_range   (string)
    @param lambda_range (string)
    @retval Iap
    """
    return QBC.GetAzimuthProfile(dat, Qcenter, dQ, azim_bin, azim_range, lambda_range)

########################################
def GetAzimuthProfileMulti(dat_list=DATList, Qcenter=0.036, dQ=0.002, azim_bin=1.0, azim_range="0.0 360.0", lambda_range="0 0"):
    """
    Calculates Azimuth Profile from DataReduction

    @param dat_list     (list)
    @param Qcenter      (float)
    @param dQ           (float)
    @param azim_bin     (float)
    @param azim_range   (string)
    @param lambda_range (string)
    @retval IapList
    """
    return QBC.GetAzimuthProfileMulti(dat_list, Qcenter, dQ, azim_bin, azim_range, lambda_range)

# Dictionary for entry the name of functions
_functions = {
            "ConvertQSpace":ConvertQSpace,
            "ConvertQSpaceRPMTCenterShift":ConvertQSpaceRPMTCenterShift,
            "ConvertQSpaceWithGravity":ConvertQSpaceWithGravity,
            "QzProjection":QzProjection,
            "SectorAverageByAngles":SectorAverageByAngles,
            "MakeRegionOfSectAve":MakeRegionOfSectAve,
            "GetAzimuthProfile":GetAzimuthProfile,
            "GetAzimuthProfileMulti":GetAzimuthProfileMulti,
            }

_functionsOrder = [
    "ConvertQSpace",
    "ConvertQSpaceWithGravity",
    "ConvertQSpaceRPMTCenterShift",
    "QzProjection",
    "SectorAverageByAngles",
    "MakeRegionOfSectAve",
    "GetAzimuthProfile",
    "GetAzimuthProfileMulti"
]



# -*- coding: utf-8 -*-
from __future__ import print_function
import os
import utsusemi.ana.Reduction.BaseCommandsInEla as BaseCom
import Manyo.Utsusemi as mu
import Manyo as mm
#####################
# Command Sequence of Data Reduction
# 2018.07.18 ver 0.4 use BaseCommandsInEla
# 2015.04.22 ver 0.3 use GetNeunetHist
# 2010.11.10 ver 0.2 use GetHistogramOfEnergyTransfer
# 2010.05.18 ver 0.1
#####################

# special reserved word for commands history
HISTORY = None
# reserved words for return value in this.
DAT = None


########################################
def CopyData(first_run=10, last_run=-1):
    """
    Copy EventData file from DAQ0 or DAQ1 into /data/XXX
    @param runno_from (int) First Run No of a measurement to be copied
    @param runno_to (int) Last Run No of a measurement to be copied
    @retval None
    """
    BaseCom.CopyData(first_run, last_run)

########################################


def GetDataOfWhite(runNo=0, startTOF=0.0, endTOF=40000.0, binTOF=100, MaskFile="mask.txt", NormFactor=1000000.0, isCT8n=False):
    """
    Load EventData and convert to histogram using corrections as below:
    - GetNeunetHist( ... )
    - NormByBeamCurrent
    - DoMask
    - SolidAngleCorrection
    @param runNo      (int)    Run Number
    @param MaskFile   (string) file name for masking
    @param NormFactor (double) normalize factor data*factor/beamCurrent. if 0, do nothing.
    @retval DAT     (ElementContainerMatrix)
    """
    mu.UtsusemiMessage('DR.GetDataOfWhite({},{},{},{},"{}",{},{})'.format(
        runNo, startTOF, endTOF, binTOF, MaskFile, NormFactor, isCT8n), True)
    return BaseCom.GetDataOfWhite(runNo, startTOF, endTOF, binTOF, MaskFile, NormFactor, isCT8n)

#########################################


def GetDataOfMonochroEi(runNo=0, Ei=0.0, Resolution=0.3, deltaE=0.0, L1=30.0, MaskFile="mask.txt", NormFactor=1000.0):
    """
    This Function is obsolete. Please forget me ...

    @param runNo    (int) Run Number
    @param Ei       (double) Incident Energy
    @param Resolution (double) Energy Resolution for tof binnings at Elastic Peak [%]
    @param deltaE   (double) Delta Energy Transfer [meV]
    @param L1       (double)  Distance between moderator to sample [m]
    @param MaskFile (string) file name for masking
    @param NormFactor (double) normalize factor data*factor/beamCurrent. if 0, do nothing.
    @retval DAT     ElementContainerMatrix as result
    """
    raise UserWarning("This Function is obsolete. Please forget me...")
    return None

#########################################


def GetDataOfMonochroEi2(runNo=0, Ei=0.0, deltaE=0.0, Emin=0.0, Emax=0.0, MaskFile="Default", NormFactor=1000000.0, offset_sec=0.0, isCT8n=True, DetEffi="None"):
    """
    Load EventData and convert to histogram using corrections as below:
    - GetNeunetHist( ... ) including MaskInfo
    - NormByBeamCurrent(DAT,NormFactor)
    - SolidAngleCorrection(DAT)
    - kikfCorrection(DAT)
    - dHWCorrection(DAT)
    - DetectorEffi(DAT, DetEffi) # if DetEffi!="None"
    @param runNo    (int) Run Number
    @param Ei       (double) Incident Energy [meV]
    @param deltaE   (double) Delta Energy Transfer [meV]
    @param Emin     (double) minimum of Energy Transfer [meV]
    @param Emax     (double) maximum of Energy Transfer [meV]
    @param MaskFile (string) file name for masking
    @param NormFactor (double) normalize factor: Intensity/beamCurrent*factor. If 0, do nothing. If minus, Intensity/abs(factor)
    @param offset_sec (float) offset second for Measurement Period to get protons
    @param isCT8n (bool) use CT8Neutron ( corrected by muon target effect ) or not
    @param DetEffi (string) File name of the detector efficiency correction data. "None" : do not correction
    @retval DAT     ElementContainerMatrix as result
    """
    mu.UtsusemiMessage('DR.GetDataOfMonochroEi2({},{},{},{},{},"{}",{},{},{},"{}")'.format(
        runNo, Ei, deltaE, Emin, Emax, MaskFile, NormFactor, offset_sec, isCT8n, DetEffi), True)
    return BaseCom.GetDataOfMonochroEi2(runNo, Ei, deltaE, Emin, Emax, MaskFile, NormFactor, offset_sec, isCT8n, DetEffi)

#########################################


def GetDataOfMonochroEi3(runNo="0", Ei=0.0, deltaE=0.0, Erange="0.0 0.0", TimeSlice="-1.0 -1.0", MaskFile="Default", NormFactor=1000000.0, offset_sec=0.0, isCT8n=True, PulseHeight="-", TimeDepBack="DARK:None", DetEffi="None"):
    """
    Load EventData and convert to histogram using corrections as below:
    - GetNeunetHist( ... ) including MaskInfo
    - NormByBeamCurrent(DAT,NormFactor)
    - SolidAngleCorrection(DAT)
    - kikfCorrection(DAT)
    - dHWCorrection(DAT)
    - DetectorEffi(DAT, DetEffi) # if DetEffi!="None"
    @param runNo    (string) Run Number(s) "<run1>[,<run2>,...]"(delimitor is comma)
    @param Ei       (double) Incident Energy [meV]
    @param deltaE   (double) Delta Energy Transfer [meV]
    @param Erange   (string) min and max of Energy Transfer [meV] "<min>,<max>"(delimitor is comma or space)
    @param TimeSlice(string) start time and end time [seconds] "<min>,<max>"(delimitor is comma or space)
    @param MaskFile (string) file name for masking
    @param NormFactor (double) normalize factor: Intensity/beamCurrent*factor. If 0, do nothing. If minus, Intensity/abs(factor)
    @param offset_sec (float) offset second for Measurement Period to get protons
    @param isCT8n (bool) use CT8Neutron ( corrected by muon target effect ) or not
    @param PulseHeigh (string) PulseHeight range "min" or "min:max", if not use "-"
    @param TimeDepBack (string) Subtruct time-independent-background "TOF:34000-36000" or "DARK:20717"
    @param DetEffi (string) File name of the detector efficiency correction data. "None" : do not correction
    @retval DAT     ElementCotainerMatix as result
    """
    mu.UtsusemiMessage('DR.GetDataOfMonochroEi3("{}",{},{},"{}","{}","{}",{},{},{},"{}","{}","{}")'.format(
        runNo, Ei, deltaE, Erange, TimeSlice, MaskFile, NormFactor, offset_sec, isCT8n, PulseHeight, TimeDepBack, DetEffi), True)
    return BaseCom.GetDataOfMonochroEi3(runNo, Ei, deltaE, Erange, TimeSlice, MaskFile, NormFactor, offset_sec, isCT8n, PulseHeight, TimeDepBack, DetEffi)

##########################################


def ToPowder(dat=DAT, startQ=-1.0, endQ=-1.0, deltaQ=0.1):
    """
    Convert data from crystal to powder

    @param dat [Def:DAT] (ElementContaienrMatrix)
    @param startQ (double) Q value of range; if minus, auto calculated
    @param endQ   (double) Q value of range; if minus, auto calculated
    @param deltaQ (double) binning of Q
    @retval ECM   (ElementContainerMatrix)
    """
    mu.UtsusemiMessage("DR.ToPowder({},{},{},{})".format(
        "<ElementContainerMatrix>", startQ, endQ, deltaQ), True)
    return BaseCom.ToPowder(dat, startQ, endQ, deltaQ)

#########################################


def MakeRunList(start_run=-1, end_run=-1, output="./run_list.txt", flag_proton=4, offset_sec=0.0, isCT8n=False, devTemp="LS340", devGonio="Gonio-1"):
    """
    Make run list
    @param start_run (int) start run number to be listed.
    @param end_run (int) start run number to be listed.
    @param output (string) output file name
    @param flag_proton (int) 0: no protons, 1:protons only, 2:kickers only, 3:both, 4:all
    @param offset_sec (float) offset second for Measurement Period to get protons
    @param isCT8n (bool) use CT8Neutron ( corrected by muon target effect ) or not
    @param devTemp (string) The instance name of temperature Control device like "LS340", "7Tmagnet", ...
    @param devGonio (string) The instance name of Gonio Control device like "Gonio", "7Tmagnet-EA"(encoder's angle), "7Tmagnet-SA"(set angle), ...
    @return None
    """
    mu.UtsusemiMessage('DR.MakeRunList({},{},"{}",{},{},{},"{}","{}")'.format(
        start_run, end_run, output, flag_proton, offset_sec, isCT8n, devTemp, devGonio), True)
    BaseCom.MakeRunList(start_run, end_run, output, flag_proton,
                        offset_sec, isCT8n, devTemp, devGonio)

#########################################


def CopyIroha2Params(first_run=10, last_run=-1, flag=0):
    """
    Copy Iroha2 Params and Log file from Iroha2 server into /data/Iroha2
    @param runno_from (int) First Run No of a measurement to be copied
    @param runno_to (int) Last Run No of a measurement to be copied
    @param flag  (int)  Flag for Copy(0) or Symbolic link(1) / default is Symbolic link or -1 for testing
    @retval None
    """
    mu.UtsusemiMessage("DR.CopyIroha2Params({},{},{})".format(
        first_run, last_run, flag), True)
    BaseCom.CopyIroha2Params(first_run, last_run, flag)


# Dictionary for entry the name of functions
_functions = {
    "CopyData": CopyData,
    "GetDataOfWhite": GetDataOfWhite,
    "GetDataOfMonochroEi": GetDataOfMonochroEi,
    "GetDataOfMonochroEi2": GetDataOfMonochroEi2,
    "GetDataOfMonochroEi3": GetDataOfMonochroEi3,
    "ToPowder": ToPowder,
    "MakeRunList": MakeRunList
}

_functionsOrder = [
    "CopyData",
    "GetDataOfWhite",
    "GetDataOfMonochroEi",
    "GetDataOfMonochroEi2",
    "GetDataOfMonochroEi3",
    "ToPowder",
    "MakeRunList"
]



from __future__ import print_function

import os
import sys
# import commands
import subprocess


class SourceSync(object):
    def __init__(self, dir1="", dir2=""):
        self.rootp1 = None
        self.rootp2 = None
        self.diff_list = []
        self.new_list1 = []
        self.new_list2 = []
        self.ext_list = [".txt", ".xrc", ".py", ".ini", ".amt", ".xml", ".cc", ".hh", ".i", ".rst", ".cpp", ".h", ".html", ".sh"]
        self.inc_list = ["Makefile", "depend"]

        self.init(dir1, dir2)

    def init(self, dir1="", dir2=""):
        if dir1 == "" or (not os.path.exists(dir1)):
            self.rootp1 = os.path.join(os.environ["HOME"], "mlf090626")
        else:
            self.rootp1 = dir1

        if dir2 == "" or (not os.path.exists(dir2)):
            self.rootp2 = os.path.join(os.environ["HOME"], "svn_chopper/chopper/mlf")
        else:
            self.rootp2 = dir2

        self.diff_list = []
        self.new_list1 = []
        self.new_list2 = []

        results1 = self._walklist(self.rootp1)
        results2 = self._walklist(self.rootp2)
        results1.sort()
        results2.sort()

        # check whether files of dir1 exist in dir2
        mv_flag = 0
        for item in results1:
            path = os.path.join(self.rootp2, item)
            if not os.path.exists(path):
                self.new_list1.append([item, mv_flag])

        # check whether files of dir2 exist in dir1
        mv_flag = 0
        for item in results2:
            path = os.path.join(self.rootp1, item)
            if not os.path.exists(path):
                self.new_list2.append([item, mv_flag])

        # do diff command on files in dir1 and dir2
        mv_flag = 1
        remove_list = []
        for item in self.new_list1:
            remove_list.append(item[0])

        for item in results1:
            if (not (item in remove_list)):
                path1 = os.path.join(self.rootp1, item)
                path2 = os.path.join(self.rootp2, item)
                cmd = "diff " + path1 + " " + path2
                diff_msg = subprocess.getoutput(cmd)

                if diff_msg != "":
                    try:
                        uni_msg = unicode(diff_msg, "utf-8")
                    except:
                        print("### ERROR : {} : Diff messages = {}".format(item, diff_msg))
                        uni_msg = diff_msg
                    self.diff_list.append([item, mv_flag, uni_msg])

    def _walklist(self, in_path):
        all_path = []
        file_list = []
        path_list = []
        ill_path = []
        for a, b, c in os.walk(in_path):
            if a.find(".svn") != -1:
                continue
            for a_c in c:
                if os.path.splitext(a_c)[1] in self.ext_list:
                    all_path.append(os.path.join(a, a_c))
                    pp = a.split(in_path)[1]
                    if len(pp) != 0:
                        if pp[0] == '/':
                            pp = pp[1:]

                    path_list.append(pp)
                    file_list.append(a_c)
                else:
                    isNotFound = True
                    for inc in self.inc_list:
                        if a_c.find(inc) >= 0:
                            all_path.append(os.path.join(a, a_c))
                            pp = a.split(in_path)[1]
                            if len(pp) != 0:
                                if pp[0] == '/':
                                    pp = pp[1:]
                            path_list.append(pp)
                            file_list.append(a_c)
                            isNotFound = False
                    if isNotFound:
                        ill_path.append(os.path.join(a, a_c))

        ret = []
        for items in zip(path_list, file_list):
            ret.append(os.path.join(items[0], items[1]))
        return ret

        """
        for items in zip(path_list,file_list):
            print ">> "+items[0]+"   "+items[1]
        print "-------"
        for item in ill_path:
            print item
        """

    def show(self, index=-1):
        if index == -1:
            print("=============================================================")
            print("     Unique Files in {}".format(self.rootp1))
            print("-------------------------------------------------------------")
            for i in range(len(self.new_list1)):
                print("{0:>3} | {1:<60} | {2:<1} |".format(i, self.new_list1[i][0], self.new_list1[i][1]))

            print("=============================================================")
            print("     Unique Files in {:^20}".format(self.rootp2))
            print("-------------------------------------------------------------")
            for i in range(len(self.new_list2)):
                print("{0:>3} | {1:<60} | {2:<1} |".format(i, self.new_list2[i][0], self.new_list2[i][1]))

            print("=============================================================")
            print("     Different Files ")
            print("-------------------------------------------------------------")
            for i in range(len(self.diff_list)):
                print("{0:>3} | {1:<60} | {2:<1} |".format(i, self.diff_list[i][0], self.diff_list[i][1]))

        else:
            self.show_comment(index)

    def show_comment(self, index):
        print("=================================")
        path1 = os.path.join(self.rootp1, self.diff_list[index][0])
        path2 = os.path.join(self.rootp2, self.diff_list[index][0])
        print("diff {} {}".format(path1, path2))

        print(self.diff_list[index][2])

    def removeDiff(self, index):
        self._changeFlag(self.diff_list, index, 0)

    def removeNew1(self, index):
        self._changeFlag(self.new_list1, index, 0)

    def removeNew2(self, index):
        self._changeFlag(self.new_list2, index, 0)

    def addDiff(self, index):
        self._changeFlag(self.diff_list, index, 1)

    def addNew1(self, index):
        self._changeFlag(self.new_list1, index, 1)

    def addNew2(self, index):
        self._changeFlag(self.new_list2, index, 1)

    def _changeFlag(self, list, index, flag):
        if flag in [0, 1]:
            if isinstance(index, int):
                list[index][1] = flag
            if isinstance(index, list):
                for i in index:
                    list[i][1] = flag

    def update1to2(self):
        print("Copying files in {} into {}.".format(self.rootp1, self.rootp2))
        item_list = self.new_list1 + self.diff_lsit
        for item in item_list:
            if item[1] != 0:
                cmd = "cp " + os.path.join(self.rootp1, item[0]) + " " + os.path.join(self.rootp2, item[0])
                print("CMD>>{}".format(cmd))
                os.system(cmd)

    def update2to1(self):
        print("Copying files in {} into {}.".format(self.rootp2, self.rootp1))
        item_list = self.new_list1 + self.diff_lsit
        for item in item_list:
            if item[1] != 0:
                cmd = "cp " + os.path.join(self.rootp1, item[0]) + " " + os.path.join(self.rootp2, item[0])
                print("CMD>>{}".format(cmd))
                os.system(cmd)


if __name__ == '__main__':
    import os
    import sys

    if len(sys.argv) == 1:
        rootp = os.path.join(os.environ["HOME"], "mlf090626")
        repop = os.path.join(os.environ["HOME"], "svn_chopper/chopper/mlf")
    if len(sys.argv) == 2:
        rootp = sys.argv[1]
        repop = os.path.join(os.environ["HOME"], "svn_chopper/chopper/mlf")
    if len(sys.argv) == 3:
        rootp = sys.argv[1]
        repop = sys.argv[2]
    print("rootp,repop={}, {}".format(rootp, repop))
    if (os.path.exists(rootp)) and (os.path.exists(repop)):
        ss = SourceSync(rootp, repop)
        ss.show()
    else:
        print("Path is invalid.")



from __future__ import print_function
#TOF_RESOLVER1 = 89770.0
#TOF_RESOLVER1 = 87770.0 #89770.0 - 50(TOF offset)*40
TOF_RESOLVER1 = 89770.0/40.0-40
#TOF_RESOLVER1 = 0.0

#TOF_RESOLVER2 = 90636.4
#TOF_RESOLVER2 = 88636.4 #90636.4 - 50(TOF offset)*40
#TOF_RESOLVER2 = 90636.4/40.0
TOF_RESOLVER2 = 0.0
import os
#import SetError as SE
import datetime
import Com
import Manyo as mm
import utsusemi.vis.VisualModuleQBase as UVM
#-----------------------------------------------------------------------------------------
def command(runNos, offsets, timeRanges, hwParam, lambdaParam, tofSource,
            startQ, endQ, deltaQ, slitwidths, useTrignets, fChop, maskInfoPath, effInfoPath):
    try:
        t1 = datetime.datetime.now()
        #import vis.M2Plot as m2p
        #import Manyo as mm
        import Cmm
        #offsets      = [-600,-420,-240,-60,120]
        #runNos       = [1965,1963,1959,1962,1966]
        #timeRanges   = ["-1.0/-1.0","-1.0/-1.0","-1.0/-1.0","-1.0/-1.0","-1.0/-1.0"]

        #hwParam      = "0.001/-0.4/0.4"
        #hwParam      = "-0.4,-0.3,-0.2,-0.1,0.0,0.1,0.2,0.3,0.4"
        #bins=[str(k*0.002 -0.4) for k in range(400)]
        #hwParam=",".join(bins)

        #lambdaParam  = "6.321/4.15"
        #tofSource    = "120/320"

        #startQ = 0.0
        #endQ   = 1.0
        #deltaQ = 0.02

        useT0ModCorr = False
        background   = 0.0
        useMonEff    = True
        
        ecms=[]
        Logformat = "{0:<9}{1:^25}{2:^17}{3:^13}\n"
        LogText = Logformat.format("Run No","timeRange", "tofSource", "useTrignet")
        print(LogText)
        ii = 0
        for runNo,offset,timeRange,slitwidth,useTrignet \
                                     in zip(runNos,offsets,timeRanges,slitwidths,useTrignets):

            if slitwidth==1:
                #slitwidth = -1
                slitwidth = 3
            elif slitwidth==0:
                slitwidth = 1
            else:
                slitwidth = -1

            if useTrignet:
                if runNo > 10511:#[TY Modified 20190409]
                    TOF_RESOLVER1=2246.9*225.0/float(fChop)

                offset = str(offset)+","+str(TOF_RESOLVER1)+","+str(TOF_RESOLVER2)
            else:
                #offset = "0.0,"+str(offset)
                #TY modified[20160427]
                t0shift = float(offset)-1.5/float(fChop)*1.0e6
                print(t0shift)
                offset = "0.0,%f" %(t0shift)
                print(offset)
                #offset = "0.0,"+str(offset-1.5/fChop*1.0e6)

            slitinfo = str(slitwidth)+"/"+str(fChop)
            print("koko67 command.py Monitor Start", runNo)
            if useTrignet:
                monec = Cmm.GetHistogramMonETrignet(runNo, str(lambdaParam), str(offset),
                                                    background, useT0ModCorr,
                                                    str(timeRange), str(tofSource), str(slitinfo))
#                                                    str(timeRange), "0/400", str(slitinfo))
            else:
                monec = Cmm.GetHistogramMonEnoTrignet(runNo, lambdaParam, offset,
                                                      background, useT0ModCorr,
                                                      timeRange, tofSource, slitinfo)
            print("koko77 command.py Monitor End", runNo)

            #Cmm.MutiplyConstant(monec, 1e-09)

            print("koko81 command.py Inela Start", runNo)
            if useTrignet:
                psdecm = Cmm.GetHistogramHWTrignet(runNo, hwParam, lambdaParam,
                                                   offset, useT0ModCorr,
                                                   timeRange, tofSource, slitinfo)
            else:
                psdecm = Cmm.GetHistogramHWnoTrignet(runNo, hwParam, lambdaParam,
                                                     offset, useT0ModCorr,
                                                     timeRange, tofSource, slitinfo)
                print(runNo, hwParam, lambdaParam, offset, useT0ModCorr,timeRange, tofSource, slitinfo)
            if maskInfoPath!="":
                #psdecm=SE.SetError(psdecm)
                maskInfoPath = str(os.path.basename(maskInfoPath))
                Cmm.DoMask(psdecm, maskInfoPath)
            print("koko95 command.py Inela end", runNo)
            #print "-->before"
            #psdecm.Put(40).Put(50).Dump()
            #Cmm.DoMaskForEachEC(psdecm)
            #print "-->after"
            #psdecm.Put(40).Put(50).Dump()
            LogText += Logformat.format(runNo, timeRange, tofSource, useTrignet)
            if ii == 0:
                ecsum = monec
                psdsum = psdecm
            else:
                print("--> Sum ec and psdecm start")
                ecsum = Com.ExecFunctionCode(Code="A+B",A=ecsum,B=monec)
                psdsum = Com.CalcContainers(dat1=psdsum,dat2=psdecm,coef1=1.0,coef2=1.0,ope="+")
                print("--> Sum ec and psdecm end")
            ii += 1
        #print "psdsum -->"
        #psdsum.Put(40).Put(50).Dump()
        ecm2 = Cmm.ILambdaCorrDNA(psdsum, ecsum, useMonEff)
        #print "ecm2--->"
        #ecm2.Put(40).Put(50).Dump()
        effInfoPath = str(effInfoPath)
        if effInfoPath!="":
            ecm3 = Cmm.SolidAngleCorrDNA(ecm2, True, False, True, "sample", DetEffDataPath = effInfoPath)
        else:
            ecm3 = Cmm.SolidAngleCorrDNA(ecm2, True, False, False, "sample")
        del ecm2
        Cmm.KiKfCorrection(ecm3) #TYAdd[20150910]
        Cmm.MutiplyConstant(ecm3, 1e3)
        
        ecaQE = Cmm.CreateQEMap(ecm3,startQ,endQ,deltaQ)
        footer = "_AMDRQE"
        Cmm.OutputECA( ecaQE, footer)
        """
        if len(ecms)>1:
            footer = "_mAMDRQE"
        """
        hwlist = []
        hwlist = hwParam.split(",")
        if len(hwlist) !=3:
            print("koko3")
            Cmm.OutputNeiECAText(ecaQE,"_neimAMDRQE")
        
        Cmm.OutputECAText( ecaQE, footer)
        Cmm.OutputECADAVEText( ecaQE, footer)
        del ecms
        del ecm3
        #Cmm.OutputECM( ecm3,footer+"ECM3")

        print(LogText)
        print("Needed time is ", (datetime.datetime.now() - t1).seconds , "s.") #TY add[20151119]
        showdat(ecaQE)
#        showdat(ecsum)
        #m2p.M2Plot(ecaQE)
        #Cmm.DoMaskForEachEC(psdecm)
        #psdecm.Put(5).Put(50).Dump()

        
    except:
        print("import Error / Manyo, MPlot")
#-----------------------------------------------------------------------------------------

def showdat(dat):
    if type(dat) ==type(mm.ElementContainer()):
        #MP.MPlot(dat)
        UVM.MPlot(dat)
    elif type(dat) == type(mm.ElementContainerArray()):
        #MP2.M2Plot(dat)
        UVM.M2PlotPlus(dat)
    else:
        print("dat type is not EC or ECA.")





#!/usr/bin/python3
# -*- coding: utf-8 -*-
"""
Utilities using on Chopper spectrometer in MLF
: 4SEASONS, AMATERAS and so on.
"""
from __future__ import print_function
from __future__ import absolute_import
import Manyo as mm
import Manyo.Utsusemi as m
from . import UtilsOnChoppers as UOC
from numpy import array, exp


class MonitorAnalysis:
    '''
    Monitor Dataからの情報の取り込み
    '''
    ################################

    def __init__(self, ECMtx=None, Ei=None, T0shift=0.0, debug=0):
        '''
        Constructor
        @param ECMtx (ElementContainerMatrix) データ。モニターカウンターのデータは
                     ECMtx(0)(0)から。
        @param Ei (Double) ユーザーが意図していた入射エネルギー
        @param T0shift (Double) TOF原点シフト量[micro-sec]
        @param debug (UInt4) debug用表示 OFF:0, ON:1
        '''
        if (ECMtx is None):
            print("You need ElementContainerMatrix of data as parameter")
            return
        SIH = mm.SearchInHeader(ECMtx)
        SIH.SearchArray("TYPE", "MONITOR")
        vec_index = SIH.PutResultIndex(0)
        if vec_index.size() == 0:
            mon_index = -1
        else:
            mon_index = vec_index[0]
        print("Monitor Array index = {}".format(mon_index))
        del SIH

        if mon_index == -1:
            print("This Element Container Matrix has no Monitor data.")
            return

        if (Ei is None):
            Ei = ECMtx(mon_index).PutHeader().PutDouble("SETEI")

        self.est_Ei = Ei
        """
        WhatIsThis = ECMtx(mon_index).PutHeader().PutString("TYPE")
        if (WhatIsThis != "MONITOR"):
            print "You need data from Monitor Counter at index 0 on ECmatrix"
            return
        """
        self.ECAry = ECMtx.Put(mon_index)
        # self.Inst = ECMtx.PutHeader().PutString("INSTRUMENT")
        self.RunNumber = ECMtx.PutHeader().PutInt4("RUNNUMBER")
        self.NumOfMonitor = self.ECAry.PutTableSize()
        self.LenOfMonitor = self.PutLengthOfMonitorFromDb()
        self.T0shift = T0shift

        self.incident_v = None
        self.incident_Ei = None
        self.OriginOfTOF = None
        self.MonitorCount = None
        self.delta_Ei = None
        self.PeakPosi = None
        self.PeakWidth = None

        self.Debug = debug
        # self.SearchPeakPosition()
        # self.CalcMonitorCount()

    ################################

    def PutLengthOfMonitorFromDb(self):
        '''
        Return distances between moderator and monitors
        現在はElementContainerのHeaderから読み取っている。
        First and second value in returned list are used
        for the calculation of incident energy.
        @retval リスト [Double, Double,..] The list of Length between moderator to Monitors
                                          in this instrument. [mm]
        '''
        LenOfMonitor = []
        for i in range(self.NumOfMonitor):
            LenOfMonitor.append(self.ECAry.Put(i).PutHeader().PutDouble("LM"))
            # LenOfMonitor = [8000.0,12000.0]

        return LenOfMonitor

    ################################
    def SearchPeakPosition(self, functype=0):
        '''
        Search Peak positions and these width

        @param functype (UInt4) To specify the fitting function.
        @retval None
        '''

        self.PeakPosi = []
        self.PeakWidth = []
        for i in range(self.NumOfMonitor):
            mon_ec = self.ECAry.Put(i)
            mon_id = mon_ec.PutHeader().PutInt4("MonitorId")
            print("Monitor ID = {}".format(mon_id))
            # This area is used for some corrections on monitor detector if you need.
            # Background, Energy dependence of efficiency, or TOF origin, and so on
            if (functype == 0):
                (peak_posi, peak_width) = self._PeakParamType1(mon_ec)
            else:
                peak_posi = None
                peak_width = None

            self.PeakPosi.append(peak_posi)
            self.PeakWidth.append(peak_width)

    ################################

    def _PeakParamType1(self, EC):
        '''
        Return parameter about fitted peak with Gaussian
        Returned peak_posi parameter is to be calclated for incident energy,
        and returned peak_width is for monitor counts.
        @param EC (ElementContainer) ElementContainer of Monitor Detector
        @retval tuple (double,double)
        '''
        u = UOC.UnitOnChoppers()

        vec_x = EC.PutX()
        vec = EC.PutY()
        init_ii = 0.0
        index = 0
        for i in range(len(vec)):
            if vec[i] > init_ii:
                init_ii = vec[i]
                index = i
        init_cc = vec_x[index]
        """
        init_ii = array(EC.PutY()).max()
        if self.Debug==1: print "init_ii",init_ii

        (ext_v, unit) = u.ToVelocity(self.est_Ei,"meV")
        if self.Debug==1: print "velocity = ",ext_v,",LM=",EC.PutHeader().PutDouble("LM")
        init_cc = EC.PutHeader().PutDouble("LM")/(ext_v/1000.0) # [mm]/[m/s]/1000.0 => [micro s]
        """
        # params = [0.0, init_ii, init_cc, 10.0] # [background, Intensity, Center Posi, gamma] of gaussians
        # [background,slope,Intensity,Center Posi,gamma] of gaussians
        params = [0.0, -1.0, init_ii, init_cc, 10.0]
        if self.Debug == 1:
            print("param for fit = {}".format(params))

        ret = self._PeakFitType1(EC, params)
        if self.Debug == 1:
            print("Peak Param = {}".format(ret))
        peak_posi = ret[2]
        peak_width = ret[3]
        return (peak_posi, peak_width)

    ################################

    def _PeakFitType1(self, EC, params):
        '''
        Do fitting on the peak in tof
        @param EC (ElementContainer) ElementContainer of Monitor Detector
        @retval list of results
        '''
        fit = UOC.FitOnChoppers()

        tof = EC.PutX()
        cnt = EC.PutY()
        xx = []
        max_yy = 0
        max_i = 0
        for i in range(tof.size() - 1):
            xx.append((tof[i] + tof[i + 1]) / 2.0)
            if cnt[i] > max_yy:
                max_yy = cnt[i]
                max_i = i
        if max_yy == 0:
            raise UserWarning("FittingError")

        xx = array(xx[(max_i - 10):(max_i + 11)])
        yy = array(cnt[(max_i - 10):(max_i + 11)])

        bb = fit.Parameter(params[0])
        ll = fit.Parameter(params[1])
        ii = fit.Parameter(params[2])
        cc = fit.Parameter(params[3])
        ss = fit.Parameter(params[4])

        def fit_func_gauss(x):
            return bb() + ll() * x + ii() * exp(-0.50 * ((x - cc()) / ss())**2)

        ret = fit.fit(fit_func_gauss, [bb, ll, ii, cc, ss], yy, xx)
        print("bb = {} \n {}".format(bb(), bb.value_err))

        return [bb(), ii(), cc(), ss()]

    ################################
    def _CalcMonitorCount(self):
        '''
        Calculate Monitor Count
        @retval
        '''
        self.MonitorCount = []
        for i in range(self.NumOfMonitor):
            tof = self.ECAry(i).PutX()
            int = self.ECAry(i).PutY()
            sum = 0.0
            limit_min = (self.PeakPosi[i] - self.PeakWidth[i])
            limit_max = (self.PeakPosi[i] + self.PeakWidth[i])
            for j in range(tof.size()):
                if (tof[j] > limit_min) and (tof[j] < limit_max):
                    sum += int[j]

            self.MonitorCount.append(sum)

    ################################
    def _CalcIncidentInform(self):
        '''
        Calculate some information of incident neutron
        Calculate incident velocity, incident energy and the origin of TOF.
        Each Value is stored as taple ( value(double),unit(string) ).

        '''

        u = UOC.UnitOnChoppers()
        if len(self.PeakPosi) == 2:
            vi = (self.LenOfMonitor[1] - self.LenOfMonitor[0]) / (self.PeakPosi[1] - self.PeakPosi[0]) * 1000.0  # [m/s]
        else:
            vi = self.LenOfMonitor[0] / (self.PeakPosi[0] - self.T0shift) * 1000.0
        self.incident_v = (vi, "m/s")
        self.incident_Ei = u.ToEnergy(vi, self.incident_v[1])
        OriginOfTOF = self.PeakPosi[0] - self.LenOfMonitor[0] / vi * 1000.0
        self.OriginOfTOF = (OriginOfTOF, "microsecond")

        v = self.LenOfMonitor[0] / (self.PeakPosi[0] - self.PeakWidth[0]) * 1000.0
        max_Ei, unit = u.ToEnergy(v, self.incident_v[1])
        v = self.LenOfMonitor[0] / (self.PeakPosi[0] + self.PeakWidth[0]) * 1000.0
        min_Ei, unit = u.ToEnergy(v, self.incident_v[1])
        self.delta_Ei = ((max_Ei - min_Ei), unit)

    ################################
    def PutIncidentEnergy(self):
        '''
        Return incident energy and its unit.

        @retval tuple (Double, "meV")
        '''
        if (self.incident_Ei is None):
            self._CalcIncidentInform()

        return self.incident_Ei

    ################################
    def PutIncidentEnergyWidth(self):
        '''
        Return incident energy and its unit.

        @retval tuple (Double, "meV")
        '''
        if (self.delta_Ei is None):
            self._CalcIncidentInform()

        return self.delta_Ei

    ################################
    def PutIncidentVelocity(self):
        '''
        Return incident velocity of neutrons ant its unit.

        @retval tuple (Double, "m/s")
        '''
        if (self.incident_v is None):
            self._CalcIncidentInform()

        return self.incident_v

    ################################
    def PutOriginOfTOF(self):
        '''
        Return TOF origin ant its unit.

        @retval tuple (Double, "microsecond")
        '''
        if (self.OriginOfTOF is None):
            self._CalcIncidentInform()

        return self.OriginOfTOF

    ################################
    def PutMonitorCount(self, monitorId=0):
        '''
        Return monitor counts
        @param monitorId (Int4) index of monitors
        @retval (double) monitor counts
        '''
        if (self.MonitorCount is None):
            self._CalcMonitorCount()

        return self.MonitorCount[monitorId]

    ################################
    def PutPeakPosi(self, monitorId=0):
        '''
        Return Peak position in TOF
        @param monitorId (Int4) index of monitors
        @retval (double) Peak position in TOF
        '''
        if (self.PeakPosi is None):
            self.SearchPeakPosition()

        return self.PeakPosi[monitorId]

    ################################
    def PutPeakWidth(self, monitorId=0):
        '''
        Return peak width in TOF
        @param monitorId (Int4) index of monitors
        @retval (double) Peak width in TOF
        '''
        if (self.PeakWidth is None):
            self.SearchPeakPosition()

        return self.PeakWidth[monitorId]



#!/usr/bin/python3
# -*- coding: utf-8 -*-

import sys
import utsusemi.SAS.ana.Reduction.TransBaseCommands as TBC
import Manyo as mm
import Manyo.SAS as ms

# reserved words for return value in this.
DAT = None
ecs = None
EC = None
ret = None
PyObj = None
Iq = None
Tr = None


#########################################
def GetTransNormByMonCounts(SmplRunNo="104000", DrctRunNo="104000", AxType="lambda", XRange="0.7 7.6 0.1", frameBoundary=0.0, CalibEffi=True ):
    """
    Monitor histogram of sample and direct followed by a transmittance calculation.
    @param SmplRunNo     (string)    Sample measurement run number
    @param DrctRunNo     (string)    Direct beam measurement run number
    @param AxType        (string)  Type must be "tof', "lambda" or "lamda2"
    @param XRange        (string) "startX endX widthX" separated with space (not comma)
    @param frameBoundary (float)
    @param CalibEffi     (bool)    Monitor-detector efficiency calibration
    @retval Tr
    """
    return TBC.GetTransNormByMonCounts(SmplRunNo, DrctRunNo, AxType, XRange, frameBoundary, CalibEffi)

#########################################
def GetTransNormByMonCountsTS(SmplRunNo="104000", DrctRunNo="104000", AxType="lambda", XRange="0.7 7.6 0.1", frameBoundary=0.0, CalibEffi=True, SmplTSlice="-1 -1", DrctTSlice="-1 -1"):
    """
    Monitor histogram of sample and direct followed by a transmittance calculation.
    @param SmplRunNo     (string)    Sample measurement run number
    @param DrctRunNo     (string)    Direct beam measurement run number
    @param AxType        (string)  Type must be "tof', "lambda" or "lamda2"
    @param XRange        (string) "startX endX widthX" separated with space (not comma)
    @param frameBoundary (float)
    @param CalibEffi     (bool)    Monitor-detector efficiency calibration
    @param SmplTSlice    (string) Time Slice parameters for Sample run
    @param DrctTSlice    (string) Time Slice parameters for Directrun
    @retval Tr
    """
    return TBC.GetTransNormByMonCountsTS(SmplRunNo, DrctRunNo, AxType, XRange, frameBoundary, CalibEffi, SmplTSlice, DrctTSlice )

#########################################
def GetTransNormByMon(SmplRunNo="104000", DrctRunNo="104000", Xrange="0.7 7.6 0.1 dL", frameBoundary="0.0", CalibEffi=True, useMon="MON1" ):
    """
    Monitor histogram of sample and direct followed by a transmittance calculation.
    @param SmplRunNo     (string)    Sample measurement run number
    @param DrctRunNo     (string)    Direct beam measurement run number
    @param Xrange        (string)  "startX endX widthX type" separated with space (not comma) : type=[dL, dL/L, tof]
    @param frameBoundary (float)
    @param CalibEffi     (bool)    Monitor-detector efficiency calibration
    @param useMon        (strng)  Upstream Monitor (MON1 or MON3)
    @retval Tr
    """
    return TBC.GetTransNormByMon(SmplRunNo, DrctRunNo, Xrange, frameBoundary, CalibEffi, useMon )

#########################################
def GetTransNormByMonTS(SmplRunNo="104000", DrctRunNo="104000", Xrange="0.7 7.6 0.1 dL", frameBoundary="0.0", CalibEffi=True, SmplTSlice="-1 -1", DrctTSlice="-1 -1", useMon="MON1", paramFiles="- -"):
    """
    Monitor histogram of sample and direct followed by a transmittance calculation.
    @param SmplRunNo     (string)    Sample measurement run number
    @param DrctRunNo     (string)    Direct beam measurement run number
    @param Xrange        (string)  "startX endX widthX type" separated with space (not comma) : type=[dL, dL/L, tof]
    @param frameBoundary (float)
    @param CalibEffi     (bool)    Monitor-detector efficiency calibration
    @param SmplTSlice    (string) Time Slice parameters for Sample run
    @param DrctTSlice    (string) Time Slice parameters for Directrun
    @param useMon        (strng)  Upstream Monitor (MON1 or MON3)
    @param paramFiles    (string) "<wiringInfo> <detectorInfo>[ <caseInfo>]
    @retval Tr
    """
    return TBC.GetTransNormByMonTS(SmplRunNo, DrctRunNo, Xrange, frameBoundary, CalibEffi, SmplTSlice, DrctTSlice, useMon, paramFiles)


# Dictionary for entry the name of functions
_functions = {
            "GetTransNormByMonCounts":GetTransNormByMonCounts,
            "GetTransNormByMonCountsTS":GetTransNormByMonCountsTS,
            "GetTransNormByMon":GetTransNormByMon,
            "GetTransNormByMonTS":GetTransNormByMonTS,
            }



# -*- coding: utf-8 -*-

import ModuleTest as ModTest


def ElementContainerArrayToText( eca, FileName ):

    f = open( FileName, "w" )
    f.close() # ファイルがすでにある場合は、上書きをする

    for i in range( eca.PutSize() ):
        target = eca( i )
        ModTest.VecToText( target.PutX(), FileName )
        ModTest.VecToText( target.PutY(), FileName )
        ModTest.VecToText( target.PutE(), FileName )



def CreateBin( start, end, width ):

    bin = []

    if not ( start < end or width < 0.0 ):
        return bin
    
    a = start

    while a <= end:
        bin.append( a )
        a = a + width
    return bin

import math

a = 0.0
b = 12.0

c = 100.0
d = 112.0

n = math.sqrt( 2.0 )
w = math.sqrt( 5.0 )

###

bin1_ln = CreateBin(a,d,n)
bin1_rw = CreateBin(a,d,w)

bin1_le = CreateBin(a,d,n)
bin1_re = CreateBin(a,d,n)

bin1_lw = CreateBin(a,d,w)
bin1_rn = CreateBin(a,d,n)

####

bin2_ln = CreateBin(a,d,n)
bin2_rw = CreateBin(b,c,w)

bin2_le = CreateBin(a,d,n)
bin2_re = CreateBin(b,c,n)

bin2_lw = CreateBin(a,d,w)
bin2_rn = CreateBin(b,c,n)

####

bin3_ln = CreateBin(b,c,n)
bin3_rw = CreateBin(a,d,w)

bin3_le = CreateBin(b,c,n)
bin3_re = CreateBin(a,d,n)

bin3_lw = CreateBin(b,c,w)
bin3_rn = CreateBin(a,d,n)

####

bin4_ln = CreateBin(a,c,n)
bin4_rw = CreateBin(b,d,w)

bin4_le = CreateBin(a,c,n)
bin4_re = CreateBin(b,d,n)

bin4_lw = CreateBin(a,c,w)
bin4_rn = CreateBin(b,d,n)

####

bin5_ln = CreateBin(b,d,n)
bin5_rw = CreateBin(a,c,w)

bin5_le = CreateBin(b,d,n)
bin5_re = CreateBin(a,c,n)

bin5_lw = CreateBin(b,d,w)
bin5_rn = CreateBin(a,c,n)

####

Lbin = [ bin1_ln, bin1_le, bin1_lw, \
         bin2_ln, bin2_le, bin2_lw, \
         bin3_ln, bin2_le, bin3_lw, \
         bin4_ln, bin2_le, bin3_lw, \
         bin5_ln, bin2_le, bin3_lw ]

Rbin = [ bin1_rw, bin1_re, bin1_rn, \
         bin2_rw, bin2_re, bin2_rn, \
         bin3_rw, bin3_re, bin3_rn, \
         bin4_rw, bin4_re, bin4_rn, \
         bin5_rw, bin5_re, bin5_rn ]

import Manyo

LEClist = Manyo.ElementContainerArray()
REClist = Manyo.ElementContainerArray()

for i in range( len( Lbin ) ):

    lbin = Lbin[ i ]
    ly = []
    le = []
    for j in range( len( lbin ) - 1 ):
        ly.append( 9.0 )
        le.append( 3.0 )
    lec = Manyo.ElementContainer()
    lec.Add( "x", lbin )
    lec.Add( "y", ly )
    lec.Add( "e", le )
    lec.SetKeys( "x", "y", "e" )
    LEClist.Add( lec )


    rbin = Rbin[ i ]
    ry = []
    re = []
    for j in range( len( rbin ) - 1 ):
        ry.append( 4.0 )
        re.append( 2.0 )
    rec = Manyo.ElementContainer()
    rec.Add( "x", rbin )
    rec.Add( "y", ry )
    rec.Add( "e", re )
    rec.SetKeys( "x", "y", "e" )
    REClist.Add( rec )


Result = Manyo.ElementContainerArray()

for i in range( LEClist.PutSize() ):
    Result.Add( LEClist.Put(i) + REClist.Put(i) )

for i in range( LEClist.PutSize() ):
    Result.Add( LEClist.Put(i) - REClist.Put(i) )

for i in range( LEClist.PutSize() ):
    Result.Add( LEClist.Put(i) * REClist.Put(i) )

for i in range( LEClist.PutSize() ):
    Result.Add( LEClist.Put(i) / REClist.Put(i) )


    
    




import os
import sys
import glob
import datetime

if len(sys.argv) > 1:
    MANYOPATH = sys.argv[1]
else:
    if "MANYO_PATH" in os.environ:
        MANYOPATH = os.environ["MANYO_PATH"]
    else:
        print("USAGE : python3 create_setenv.py <MANYO_PATH>")
        print("  Arguments:")
        print("            <MANYO_PATH> : Path of MLF software root like /opt/mlfsoft")
        sys.exit(0)

PYVERMAJ = sys.version_info.major
PYVERMIN = sys.version_info.minor
PYVERMIC = sys.version_info.micro

# search pypath
pypath_s = "/opt/homebrew/Cellar/python@{0}.{1}/{0}.{1}.{2}*".format(PYVERMAJ, PYVERMIN, PYVERMIC)
v = glob.glob(pypath_s)
if len(v) == 0:
    print("Not found python3")
    sys.exit(-1)
v.sort()
pypath = os.path.join(v[-1], "Frameworks/Python.framework/Versions/Current")

# search libomp version
l = glob.glob("/opt/homebrew/Cellar/libomp/*")
if len(l) == 0:
    print("Not found libomp")
    sys.exit(-1)
l.sort()
libomp_path = l[-1]

d = datetime.datetime.now().strftime('%Y-%m-%d %H:%M:%S')
if os.path.exists(pypath):
    print("OK {}".format(pypath))
    cont = "# Created by create_setenv.py at {}\n".format(d)
    cont += "# Manyo Library Path like /opt/mlfsoft/manyo\n"
    cont += "unset PREFIX\n"
    cont += "export MANYO_PATH={}\n".format(MANYOPATH)
    cont += "\n"
    cont += ("BREWPYPATH="+pypath+"\n")
    cont += "BREWPYINCPATH=${BREWPYPATH}/include/python"
    cont += "{}.{}\n".format(PYVERMAJ, PYVERMIN)
    cont += "BREWPYLIBPATH=${BREWPYPATH}/lib\n"
    cont += "\n"
    cont += "# LIBRARY_PATH for compile\n"
    cont += "export LIBRARY_PATH=${MANYO_PATH}/lib:\\\n"
    cont += "/opt/homebrew/lib:\\\n"
    cont += "/opt/homebrew/opt/openssl/lib:\\\n"
    cont += "/opt/homebrew/opt/zlib/lib:\\\n"
    cont += libomp_path+"/lib:\\\n"
    cont += "${BREWPYLIBPATH}\n"
    cont += "\n"
    cont += "export DYLD_LIBRARY_PATH=${LIBRARY_PATH}\n\n"
    cont += "# INCLUDE_PATH for compile\n"
    cont += "export CPLUS_INCLUDE_PATH=${MANYO_PATH}/include:\\\n"
    cont += "${MANYO_PATH}/include/boost:\\\n"
    cont += "${MANYO_PATH}/include/nexus:\\\n"
    cont += "/opt/homebrew/include/:\\\n"
    cont += "${BREWPYINCPATH}:\\\n"
    cont += "/opt/homebrew/opt/openssl/include:\\\n"
    cont += libomp_path+"/include\n"
    cont += "\n"
    cont += "# Installation path and Python version\n"
    cont += "export INSTALL_PATH=${MANYO_PATH}\n"
    cont += "export PYTHONPATH=${MANYO_PATH}\n"
    cont += ("export PyVer=" + "{}.{}\n".format(PYVERMAJ, PYVERMIN))
    fo = open("setenv.sh", "w")
    fo.write(cont)
    fo.close()

    # Make _setEnvMac
    cont = "# Set environment for macOS\n"
    cont += "# Created by create_setenv.py at {}\n".format(d)
    cont += "# Set path to package root directory\n"
    cont += "# echo '======== _SetEnvUbuntuForMac ==========='\n"
    cont += "_DIR_=$(cd $(dirname $0); pwd)\n"
    cont += "export MLFSOFT_PATH=${_DIR_}/..\n"
    cont += "export MANYO_PATH=${MLFSOFT_PATH}/manyo\n"
    cont += "PYPATH_UTSUSEMI=${MLFSOFT_PATH}/python-utsusemi\n"
    cont += "export UTSUSEMI_BASE_DIR=${PYPATH_UTSUSEMI}/utsusemi\n"
    cont += "PYBIN=${MANYO_PATH}/bin:${UTSUSEMI_BASE_DIR}/bin\n"
    cont += "export PYTHONPATH=${MANYO_PATH}:${PYPATH_UTSUSEMI}:${UTSUSEMI_BASE_DIR}/facade:${UTSUSEMI_BASE_DIR}/${UTSUSEMI_INST_CODE}/facade:\n"
    cont += "export PATH=${PYBIN}:${PATH}\n"
    cont += "\n"
    cont += "# Add LIBRARY_PATH\n"
    cont += ("BREWPYPATH="+pypath+"\n")
    cont += "BREWPYINCPATH=${BREWPYPATH}/include/python"
    cont += "{}.{}\n".format(PYVERMAJ, PYVERMIN)
    cont += "BREWPYLIBPATH=${BREWPYPATH}/lib\n"
    cont += "LIBRARY_PATH=${MANYO_PATH}/lib:/opt/homebrew/lib:/opt/homebrew/opt/openssl/lib:/opt/homebrew/opt/zlib/lib:"
    cont += libomp_path+"/lib:${BREWPYLIBPATH}\n"
    cont += "DYLD_LIBRARY_PATH=${LIBRARY_PATH}\n"
    cont += "export LIBRARY_PATH\n"
    cont += "export DYLD_LIBRARY_PATH\n"
    fo = open("_setEnvMac", "w")
    fo.write(cont)
    fo.close()
else:
    print("FAILURE {}".format(pypath))

